############################################################################################################
# VaRank 1.3                                                                                               #
#                                                                                                          #
# VaRank: a simple and powerful tool for ranking genetic variants                                          #
#                                                                                                          #
# Copyright (C) 2015 Veronique Geoffroy (veronique.geoffroy@inserm.fr)                                     # 
#                    Jean Muller (jeanmuller@unistra.fr)                                                   # 
#                                                                                                          #
# Please cite the following article:                                                                       #
# Geoffroy V.*, Pizot C.*, Redin C., Piton A., Vasli N., Stoetzel C., Blavier A., Laporte J. and Muller J. #
# VaRank: a simple and powerful tool for ranking genetic variants.                                         #
# PeerJ. 2015 (10.7717/peerj.796)			       		                                   #
#                                                                                                          #
# This is part of VaRank source code.                                                                      #
#                                                                                                          #
# This program is free software; you can redistribute it and/or                                            #
# modify it under the terms of the GNU General Public License                                              # 
# as published by the Free Software Foundation; either version 3                                           # 
# of the License, or (at your option) any later version.                                                   #
#                                                                                                          #
# This program is distributed in the hope that it will be useful,                                          # 
# but WITHOUT ANY WARRANTY; without even the implied warranty of                                           #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                                            #
# GNU General Public License for more details.                                                             #
#                                                                                                          #
# You should have received a copy of the GNU General Public License                                        #
# along with this program; If not, see <http://www.gnu.org/licenses/>.                                     #
############################################################################################################

proc checkAlamut {} {

    global g_VaRank


    puts "...checking Alamut Batch ([clock format [clock seconds] -format "%B %d %Y - %H:%M"])"


    if {![file exists $g_VaRank(vcfDir)/Alamut]} {
	file mkdir $g_VaRank(vcfDir)/Alamut
    }
    ## Creating an input test for Alamut Batch
    ReplaceTextInFile "id\t11\t66293652\tT\tG" $g_VaRank(vcfDir)/Alamut/test.input

    ## Checking if the licence is accepted
    set test 0
    foreach f [glob -nocomplain $g_VaRank(alamutDir)/alamut-*.ini] {
	if {[file exists $f]} {
	    if {[regexp "LicenseAccepted=true" [ContentFromFile $f]]} {
		set test 1
	    }
	}
    }

    if {! $test} {
	puts "##############################################################"
	puts "\t...Alamut Batch License agreement is apparently no accepted ($f)."
	puts "\t   Please, run Alamut Batch on a single example and accept the terms of licence before running VaRank."
	puts "\t   You can run the two following command lines:"
	puts "\t   cd $g_VaRank(vcfDir)/Alamut"
	puts "\t   alamut-batch --assbly $g_VaRank(alamutHumanDB) --in test.input --ann test.ann --unann test.unann --alltrans --outputannonly --ssIntronicRange 2"
	puts "\t   EXIT of VaRank!"
	puts "##############################################################"
	exit
    }

    ## Checking Alamut Batch running and Alamut Batch version
    set outputFile "$g_VaRank(vcfDir)/Alamut/test.output"
    set alamutCmd "$g_VaRank(alamutDir)/alamut-batch --assbly $g_VaRank(alamutHumanDB) --in $g_VaRank(vcfDir)/Alamut/test.input --ann $g_VaRank(vcfDir)/Alamut/test.ann --unann $g_VaRank(vcfDir)/Alamut/test.unann --alltrans --outputannonly --ssIntronicRange 2 >& $outputFile"	  

    if {[catch {eval exec $alamutCmd} Message]} {	
	puts "\nAlamut unexpected stop\n$Message" 
	
	set What "Sorry. Access denied: key .* in use by"
	if {[regexp $What $Message]} {
	    puts "##############################################################"
	    puts "\t...Alamut Batch access denied because key in use."
	    puts "\t   Please rerun VaRank when Alamut Batch will be available."
	    puts "\t   EXIT of VaRank!"
	    puts "##############################################################"
	    exit
	} 
	
	set What  "expired license"
	if {[regexp $What $Message]} {	    
	    puts "##############################################################"
	    puts "\t...Alamut Batch expired license!"
	    puts "\t   Please make sure your Alamut Batch license is still active."
	    puts "\t   EXIT of VaRank!"
	    puts "##############################################################"
	    exit
	} 	
    } else {
	set A 1
	set B 4
	set C 2
	if {[regexp "alamut-.*? version (\[0-9\]+)\\.(\[0-9\]+)\\.(\[0-9\]+)" [ContentFromFile $outputFile] match vA vB vC]} {
	    set test 0
	    if {$vA != $A} {
		set test 1
	    } elseif {$vB != $B} {
		set test 1
	    } elseif {$vC > $C} {
		set test 1
	    }
	    if {$test} {
		puts "############################################################################"
		puts "\t...WARNING: VaRank $g_VaRank(Version) is compatible with Alamut Batch 1.4.2"
		puts "\t   You are using Alamut Batch $vA.$vB.$vC"
		puts "############################################################################"
	    }
	}
    }
}

  
## Use of the g_vcfINFOS global variable to create the Alamut input file (one file for all patients)
proc createAlamutInputFile {} {

    global g_vcfINFOS
    global g_VaRank

    set VCFdir $g_VaRank(vcfDir)
    set AlamutInputFile "$VCFdir/Alamut/AlamutInputFile_all.txt"

    if {![file exists "$VCFdir/Alamut"]} {file mkdir "$VCFdir/Alamut"}

    puts "...creation of the alamut input file ($AlamutInputFile) ([clock format [clock seconds] -format "%B %d %Y - %H:%M"])"
    file delete -force $AlamutInputFile

    set nbSNV 0
    set AlamutText ""

    foreach ID [set g_vcfINFOS(L_IDs)] {
	# If ALT=*, then variant is not wrote into the input file for annotation.
	if {[regexp "_\\*$" $ID]} {continue}

	append AlamutText "$ID\t[join [lrange $g_vcfINFOS($ID) 0 3] "\t"]\n"		
	incr nbSNV
    }	
    if {[info exists g_VaRank(DEBUG)]} {$puts "\t...$nbSNV variations created"}

    regsub "\n$" $AlamutText "" AlamutText
    WriteTextInFile $AlamutText $AlamutInputFile

    return
}


## Return 0 if all the input file variations have been analysed or if the input file is empty.
## Else, return 1 and create an TmpAlamutInputFile with the variations not yet analysed.
proc AlamutIsNotCompleted {inputFile annFile unannFile outputFile {segmentation 0}} {
    
    regsub "InputFile_all.txt" $inputFile "CannotLoadGene.txt" cannotloadgeneFile
    
    if {[file size $inputFile] == 0} {
	puts "\t...Empty input file for Alamut: $inputFile"
	return 0
    }

    regsub "InputFile_all.txt" $inputFile "InputFile_remainingID.tmp" TmpAlamutInputFile

    file delete -force $TmpAlamutInputFile

    ## Load into lIDana the ID already analysed by Alamut.
    set lIDana {}

    if {[file exists $annFile]} {
	foreach L [LinesFromFile $annFile] {
	    set ID [lindex $L 0]
	    if {![info exists TabIDana($ID)]} {set TabIDana($ID) 1;lappend lIDana $ID} else {continue}
	}
    }
    if {[file exists $unannFile]} {
	foreach L [LinesFromFile $unannFile] {
	    set ID [lindex $L 0]
	    if {![info exists TabIDana($ID)]} {set TabIDana($ID) 1;lappend lIDana $ID} else {continue}
	}
    } 
    if {$lIDana == ""} {
	if {$segmentation} {
	    WriteTextInFile [join [lrange [LinesFromFile $inputFile] 1 end] "\n"] $TmpAlamutInputFile
	    WriteTextInFile "[lindex [LinesFromFile $inputFile] 1]: child killed: segmentation violation" $unannFile
	} else {
	    file copy -force $inputFile $TmpAlamutInputFile
	}
	return 1
    }

    ## Load into TabIDinvalid the ID that need to be updated by Alamut developer:
    ## Genes defined with "Invalid data while loading gene" or "Cannot load gene" must be requested by mail to the Alamut developers.
    ## These ID don't work with Alamut, we do not put them in the inputfile.

    if {[file exists $outputFile]} {
	foreach L [LinesFromFile $outputFile] {
	    #puts $L
	    if {[regexp "Cannot load gene (.*)" $L match gene]} {
		regsub ":" $L "" L
		#Room for improvment, use lsearch -index 0 or add the identifiers from lIDinvalid into a Tab like for lIDana
		#JEAN
		#lappend lIDinvalid {[lindex $L 0] $gene}
		set ID [lindex $L 0]
		if {![info exists TabIDinvalid($ID)]} {set TabIDinvalid($ID) $gene} else {continue}

		#lappend lIDinvalid "[lindex $L 0] $gene]"
	    }
	}
    }
    #set lIDinvalid [lsort -unique $lIDinvalid]


    set test 0
    ## Some ID (With Knome for example) are too long to be treated by a lsearch:
    ## "couldn't compile regular expression pattern: out of memory"
    ## They are not analyzed and are written in $unannFile.strange
    file delete -force $unannFile.strange

    ## TmpAlamutInputFile creation:
    file delete -force $cannotloadgeneFile
    foreach L [LinesFromFile $inputFile] {
	if {$L == ""} {continue}

	set ID [lindex $L 0]
	
	if {[info exists TabIDinvalid($ID)]} {
	    WriteTextInFile "Cannot load gene [set TabIDinvalid($ID)]!!! $ID not analysed" $cannotloadgeneFile
	    #WriteTextInFile "Cannot load gene [lindex [lindex $lIDinvalid $i] end]!!! $ID not analysed" $cannotloadgeneFile
	    continue
	}
	
	if {![info exists TabIDana($ID)]} {
	    if {$segmentation} {
		set segmentation 0
		WriteTextInFile "$L: child killed: segmentation violation" $unannFile
		continue
	    }
	    set test 1
	    WriteTextInFile $L $TmpAlamutInputFile

	}
    }
    
    if {[file exists $cannotloadgeneFile]} {
	WriteTextInFile "\nPlease, ask for an update for these genes to the Alamut Batch developers." $cannotloadgeneFile
    }

    return $test
}

#################################################################################################
# OUTPUT:
#    - $VCFdir/Alamut/AlamutAnnotations_all.txt
#    - $VCFdir/Alamut/AlamutUnannotated_all.txt
#    - $VCFdir/Alamut/AlamutOutput_all.txt
#
# RETURN :
#    - return "1" if Alamut has been run for all the variations
#    - else "exit" (if alamut has been run 10 times and is still not completed)
#     	 	   (or if alamut license expired)
#################################################################################################
proc runAlamut {} {

    global g_VaRank

    set VCFdir $g_VaRank(vcfDir)
    set AlamutInputFile "$VCFdir/Alamut/AlamutInputFile_all.txt"
    regsub "AlamutInputFile"   $AlamutInputFile "AlamutAnnotations"  annFile
    regsub "AlamutInputFile"   $AlamutInputFile "AlamutUnannotated"  unannFile
    regsub "AlamutInputFile"   $AlamutInputFile "AlamutOutput"       outputFile
    regsub "InputFile_all.txt" $AlamutInputFile "CannotLoadGene.txt" cannotloadgeneFile

    file delete -force $outputFile
    file delete -force $cannotloadgeneFile

    ## Running Alamut for the first time if the output files don't exist.
    #
    puts "...running Alamut Batch ([clock format [clock seconds] -format "%B %d %Y - %H:%M"])"

    if {![file exists $annFile] && ![file exists $unannFile]} {
	set DateDuJour [clock seconds]
	ReplaceTextInFile "Alamut Batch started: [clock format $DateDuJour -format "%B %d %Y - %H:%M"]" $outputFile

	set doitagain 1
	while {$doitagain} {

	    if {([info exists g_VaRank(hgmdUser)] && $g_VaRank(hgmdUser) != "") && ([info exists g_VaRank(hgmdPasswd)] && $g_VaRank(hgmdPasswd) != "")} {
		set alamutCmd "$g_VaRank(alamutDir)/alamut-batch --assbly $g_VaRank(alamutHumanDB) --in $AlamutInputFile --ann $annFile --unann $unannFile --alltrans --outputannonly --hgmdUser $g_VaRank(hgmdUser) --hgmdPasswd $g_VaRank(hgmdPasswd) --ssIntronicRange 2 >>& $outputFile"
	    } else {
		set alamutCmd "$g_VaRank(alamutDir)/alamut-batch --assbly $g_VaRank(alamutHumanDB) --in $AlamutInputFile --ann $annFile --unann $unannFile --alltrans --outputannonly --ssIntronicRange 2 >>& $outputFile"
	    }

	    #set  CODE [catch {exec $FILE_cmd > $OutputFileStat} Message]
	    #if {$CODE != "0"} {puts stderr "stderr $Message"}

	    if {[catch {eval exec $alamutCmd} Message]} {
		## Alamut has implemented a token system so that we can run only x instance(s) at once.
		## Else we get the error message "Sorry. ... Access denied."

		WriteTextInFile "\nAlamut unexpected stop\n$Message" $outputFile

		set ErrorLines ""
		if {[file exists $outputFile]} {
		    set ErrorLines [join [LinesFromFile $outputFile] " "]
		}
		#file delete -force $outputFile
		
		set What "Sorry. Access denied: key .* in use by"
		if {[regexp $What $Message]||[regexp $What $ErrorLines]} {
		    puts "\t...Alamut Batch access denied because key in use, waiting."
		    after 600000; # on attend 10 min qu'une instance se libre
		} else {
		    set doitagain 0
		}

		set What  "expired license"
		if {[regexp $What $Message]||[regexp $What $ErrorLines]} {
		    #Sorry. Access denied: expired license

		    puts "##############################################################"
		    puts "\t...Alamut Batch license expired!"
		    puts "\t   Please make sure your Alamut Batch license is still active."
		    puts "\t   EXIT of VaRank!"
		    puts "##############################################################"
		    exit
		} 

		set What "HGMD Pro authentication failed"
		if {[regexp $What $Message]||[regexp $What $ErrorLines]} {
		    #HGMD Pro authentication failed: Error: Subscription has expired.

		    puts "############################################################################"
		    puts "\t...HGMD pro expired license!"
		    puts "\t   Please make sure that your HGMD pro license is still active."
		    puts "\t   EXIT of VaRank!"
		    puts "############################################################################"
		    exit
		} 
	    } else {
		set doitagain 0
	    }
	}
	set DateDuJour [clock seconds]
	WriteTextInFile "Alamut Batch finished: [clock format $DateDuJour -format "%B %d %Y - %H:%M"]\n\n" $outputFile
    } elseif {![AlamutIsNotCompleted $AlamutInputFile $annFile $unannFile $outputFile]} {
	puts "\t...Alamut Batch has already analysed all variations, continue ([clock format [clock seconds] -format "%B %d %Y - %H:%M"])"
	return 1
    }
    
    if {![info exists g_VaRank(AlamutHeader)]} {
	#BUG HEADER PROBLEM one way to solve it

	if {[file exists $annFile]} {
	    set Header [FirstLineFromFile $annFile]
	    if {![regexp {^#} $Header]} {puts "\t...WARNING Trying to save the Alamut Batch header but already missing"} else {set g_VaRank(AlamutHeader) $Header}
	}
    }
    
    ## Rerun Alamut until all variations have been analysed (max executed 10 times)
    #
    set i 0

    regsub "InputFile_all.txt" $AlamutInputFile "InputFile_remainingID.tmp" TmpAlamutInputFile

    while {[AlamutIsNotCompleted $AlamutInputFile $annFile $unannFile $outputFile] && $i < 10} {
	puts "\t...checking if Alamut Batch has already run on all ([clock format [clock seconds] -format "%B %d %Y - %H:%M"])"
	puts "\t...Alamut Batch didn't finish to analyse all variations. New run is launched ([clock format [clock seconds] -format "%B %d %Y - %H:%M"])"
	set DateDuJour [clock seconds]
	WriteTextInFile "Alamut Batch started: [clock format $DateDuJour -format "%B %d %Y - %H:%M"]" $outputFile

	set doitagain 1
	set segmentation 0
	while {$doitagain} {

	    if {([info exists g_VaRank(hgmdUser)] && $g_VaRank(hgmdUser) != "") && ([info exists g_VaRank(hgmdPasswd)] && $g_VaRank(hgmdPasswd) != "")} {
		set alamutCmd "$g_VaRank(alamutDir)/alamut-batch --assbly $g_VaRank(alamutHumanDB) --in $TmpAlamutInputFile --ann $annFile.tmp --unann $unannFile.tmp --alltrans --outputannonly --hgmdUser $g_VaRank(hgmdUser) --hgmdPasswd $g_VaRank(hgmdPasswd) --ssIntronicRange 2 >>& $outputFile"
	    } else {
		set alamutCmd "$g_VaRank(alamutDir)/alamut-batch --assbly $g_VaRank(alamutHumanDB) --in $TmpAlamutInputFile --ann $annFile.tmp --unann $unannFile.tmp --alltrans --outputannonly --ssIntronicRange 2 >>& $outputFile"
	    }

	    WriteTextInFile "\n$alamutCmd" $outputFile
	    if {[catch {eval exec $alamutCmd} Message]} {
		WriteTextInFile "\nAlamut unexpected stop\n$Message" $outputFile

		set ErrorLines ""
		if {[file exists $outputFile]} {
		    set ErrorLines [join [LinesFromFile $outputFile] " "]
		}
		
		set What  "expired license"
		if {[regexp $What $Message]||[regexp $What $ErrorLines]} {
		    #Sorry. Access denied: expired license

		    puts "##############################################################"
		    puts "\t...Alamut Batch expired license!"
		    puts "\t   Please make sure your Alamut Batch license is still active."
		    puts "\t   EXIT of VaRank!"
		    puts "##############################################################"
		    exit
		} 

		set What "HGMD Pro authentication failed"
		if {[regexp $What $Message]||[regexp $What $ErrorLines]} {
		    #HGMD Pro authentication failed: Error: Subscription has expired.

		    puts "############################################################################"
		    puts "\t...HGMD pro license seems to be expired!"
		    puts "\t   Please make sure that your HGMD pro license is still active."
		    puts "\t   EXIT of VaRank!"
		    puts "############################################################################"
		    exit
		} 

		set What "child killed: segmentation violation"
		if {[regexp $What $Message]||[regexp $What $ErrorLines]} {
		    #Some ID caused "segmentation violation" in the alamut program only on debian. Not successfully debbugged by alamut.

		    puts "############################################################################"
		    puts "\t...Alamut Batch segmentation violation"
		    puts "############################################################################"
		    
		    # We will remove the bad ID from the input file
		    set segmentation 1

		    if {[file exists $annFile.tmp] && [file size $annFile.tmp] != 0} {

			set  infosAnn [lrange [LinesFromFile $annFile.tmp] 1 end]
			if {$infosAnn != ""} {
			    WriteTextInFile [join $infosAnn "\n"] $annFile
			}
		    }
		    if {[file exists $unannFile.tmp] && [file size $unannFile.tmp] != 0} {
			WriteTextInFile [join [LinesFromFile $unannFile.tmp] "\n"] $unannFile
		    }
		    
		    file delete -force $annFile.tmp
		    file delete -force $unannFile.tmp
		    WriteTextInFile [join [LinesFromFile $outputFile] "\n"] $outputFile.$i
		    file delete -force $outputFile

		    AlamutIsNotCompleted $AlamutInputFile $annFile $unannFile $outputFile $segmentation
		    continue
		} 

		set What "Sorry. Access denied: key .* in use by"
		if {[regexp $What $Message]||[regexp $What $ErrorLines]} {
		    after 600000; # on attend 10 min qu'une instance se libre
		    if {[file exists $annFile.tmp] && [file size $annFile.tmp] != 0} {
			set  infosAnn [lrange [LinesFromFile $annFile.tmp] 1 end]
			if {$infosAnn != ""} {
			    WriteTextInFile [join $infosAnn "\n"] $annFile
			}
		    }
		    if {[file exists $unannFile.tmp] && [file size $unannFile.tmp] != 0} {
			WriteTextInFile [join [LinesFromFile $unannFile.tmp] "\n"] $unannFile
		    }
		    
		    file delete -force $annFile.tmp
		    file delete -force $unannFile.tmp
		    WriteTextInFile [join [LinesFromFile $outputFile] "\n"] $outputFile.$i
		    file delete -force $outputFile
		} else {
		    set doitagain 0
		}
	    } else {
		WriteTextInFile "\n$Message" $outputFile
		set doitagain 0
	    }
	}
	set DateDuJour [clock seconds]
	WriteTextInFile "Alamut Batch finished: [clock format $DateDuJour -format "%B %d %Y - %H:%M"]\n\n" $outputFile

	incr i
	if {[file exists $annFile.tmp] && [file size $annFile.tmp] != 0} {

	    set  infosAnn [lrange [LinesFromFile $annFile.tmp] 1 end]
	    if {$infosAnn != ""} {
		WriteTextInFile [join $infosAnn "\n"] $annFile
	    }
	}
	if {[file exists $unannFile.tmp] && [file size $unannFile.tmp] != 0} {
	    WriteTextInFile [join [LinesFromFile $unannFile.tmp] "\n"] $unannFile
	}
	file delete -force $annFile.tmp
	file delete -force $unannFile.tmp
    }

    if {$i == 10} {
	# Alamut did not run on all variations
	if {[file exists $cannotloadgeneFile]} {
	    puts "[ContentFromFile $cannotloadgeneFile]\n"
	}

	puts "##############################################################"
	puts "\t   WARNING: Alamut Batch is not finished after 10 run!"
	puts "\t   EXIT of VaRank!"
	puts "##############################################################"
	exit
    }

    ## Alamut has been run for all the variations
    if {[file exists $cannotloadgeneFile]} {
	puts "[ContentFromFile $cannotloadgeneFile]\n"
    }

    return 1
}

proc parseAlamutFile {{afterPPH 0}} {

    global g_ANNOTATION
    global g_VaRank
    global g_vcfINFOS

    set VCFdir $g_VaRank(vcfDir)
    set annFile "$VCFdir/Alamut/AlamutAnnotations_all.txt"

    ## reminder: backup of the headline into g_ANNOTATION(#id)

    if {! $afterPPH} {
	puts "...parsing Alamut Batch results ([clock format [clock seconds] -format "%B %d %Y - %H:%M"])"
    }

    set First 1
    foreach L [LinesFromFile $annFile] {

	set ID [lindex $L 0]

	#Analyzing the alamut header
	#
	if {$First} {
	    set First 0
	    if {![regexp {^#} $L]} {
		if {[info exists g_VaRank(AlamutHeader)] && [set g_VaRank(AlamutHeader)]!=""} {
		    set ID [lindex $L 0]
		    lappend g_ANNOTATION($ID) "$L"
		    puts "\t...WARNING Alamut Batch output file seems to miss the header. Rescue done!"
		} else {
		    puts "\t...WARNING Alamut Batch output file seems to miss the header. Exit of VaRank!"
		    exit
		}
	    }
	} else {
	    # Add selectively to load only variants from the patients analyzed
	    if {![info exists g_vcfINFOS($ID)]} {continue}
	}

	# Replacing the empty columns by NA
	#   Note regsub is faster
	while {[regexp "\t\t" $L]} {
	    #set L [string map {"\t\t" "\tNA\t"} $L]
	    regsub -all "\t\t" $L "\tNA\t" L
	}
	regsub "\t$" $L "\tNA" L

	## ID corresponds to 1 genetic variant.
	## There may be multiple transcripts per genetic variant (strand +, strand - different splicing).
	## This need to be handled specifically later to ensure coherence of the specific data for each isoform
	lappend g_ANNOTATION($ID) "$L"
    }

    return 
}

