############################################################################################################
# VaRank 1.3                                                                                               #
#                                                                                                          #
# VaRank: a simple and powerful tool for ranking genetic variants                                          #
#                                                                                                          #
# Copyright (C) 2015 Veronique Geoffroy (veronique.geoffroy@inserm.fr)                                     #
#                    Jean Muller (jeanmuller@unistra.fr)                                                   #
#                                                                                                          #
# Please cite the following article:                                                                       #
# Geoffroy V.*, Pizot C.*, Redin C., Piton A., Vasli N., Stoetzel C., Blavier A., Laporte J. and Muller J. #
# VaRank: a simple and powerful tool for ranking genetic variants.                                         #
# PeerJ. 2015 (10.7717/peerj.796)                                                                          #
#                                                                                                          #
# This is part of VaRank source code.                                                                      #
#                                                                                                          #
# This program is free software; you can redistribute it and/or                                            #
# modify it under the terms of the GNU General Public License                                              #
# as published by the Free Software Foundation; either version 3                                           #
# of the License, or (at your option) any later version.                                                   #
#                                                                                                          #
# This program is distributed in the hope that it will be useful,                                          #
# but WITHOUT ANY WARRANTY; without even the implied warranty of                                           #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                                            #
# GNU General Public License for more details.                                                             #
#                                                                                                          #
# You should have received a copy of the GNU General Public License                                        #
# along with this program; If not, see <http://www.gnu.org/licenses/>.                                     #
############################################################################################################


proc createSnpEffInputFile {} {

    global g_VaRank

    ## Keep SnpEff input files already existing
    ## No deletion


    ## Removing snpEff input files not already annotated
    foreach inputFile [glob -nocomplain $g_VaRank(vcfDir)/SnpEff/InputDir/*.vcf] {
	regsub "InputDir" $inputFile "OutputDir" annFile
	regsub ".vcf" $annFile ".varType.dbsnp.dbnsfp.phastCons.vcf" annFile
	if {![file exists $annFile]} {file delete -force $inputFile}
    }
    

    ## Creation of the SnpEff input files (containing non redundant variants)
    #
    puts "...creation of the SnpEff input files ([clock format [clock seconds] -format "%B %d %Y - %H:%M"])"
    file mkdir $g_VaRank(vcfDir)/SnpEff
    file mkdir $g_VaRank(vcfDir)/SnpEff/Input


    ## If some SnpEff outputFiles already exists, their ID should not be reported in the input file (because already annotated)
    ## Load into lIDana the ID already analysed by SnpEff
    set lIDana {}
    foreach annFile [glob -nocomplain $g_VaRank(vcfDir)/SnpEff/Output/*.varType.dbsnp.dbnsfp.phastCons.vcf] {
	foreach L [LinesFromFile $annFile] {
	    if {[regexp "^#" $L]} {continue}
	    lappend lIDana "[lindex $L 0]_[lindex $L 1]_[lindex $L 3]_[lindex $L 4]"		
	}
    }	

    set i 1
    # Warning: Check the incrementation ($i) for inputfile names (should not be the same as already existed output file)
    while {[file exists $g_VaRank(vcfDir)/SnpEff/Output/allinone.$i.eff.varType.dbsnp.dbnsfp.phastCons.vcf]} {
	incr i
    }
    set nbL 0
    set snpeffInputFile $g_VaRank(vcfDir)/SnpEff/Input/allinone.$i.vcf
    ReplaceTextInFile "#CHROM\tPOS\tID\tREF\tALT\tQUAL\tFILTER\tINFO\tFORMAT\ttutu" $snpeffInputFile

    set lVariantLines {}
    set allIDfromVCFs {}
    foreach vcfFile [glob -nocomplain $g_VaRank(vcfDir)/*.vcf $g_VaRank(vcfDir)/*.vcf.gz] {
	puts "\t...parsing $vcfFile"

        if {[regexp ".vcf.gz$" $vcfFile]} {
            set lLines [LinesFromGZFile $vcfFile]
        } else {
            set lLines [LinesFromFile $vcfFile]
        }
        foreach L $lLines {
            if {[regexp "^##" $L]} {continue}
            if {[regexp "^#CHROM" $L]} {
                set Ls [split $L "\t"]
                set i_chr    [lsearch -exact $Ls "#CHROM" ]; if {$i_chr    != 0} {puts "Bad header line syntax: $L\nExit"; exit}
                set i_pos    [lsearch -exact $Ls "POS"];     if {$i_pos    != 1} {puts "Bad header line syntax: $L\nExit"; exit}
                set i_id     [lsearch -exact $Ls "ID"];      if {$i_id     != 2} {puts "Bad header line syntax: $L\nExit"; exit}
                set i_ref    [lsearch -exact $Ls "REF"];     if {$i_ref    != 3} {puts "Bad header line syntax: $L\nExit"; exit}
                set i_alt    [lsearch -exact $Ls "ALT"];     if {$i_alt    != 4} {puts "Bad header line syntax: $L\nExit"; exit}
             
		continue
            }

            if {$L == ""} {continue}

            ## If there is only normal hom in the line (GT = 0|0 or 0/0), so there isn't mutation to analyse by VaRank
            if {![regexp "1/0|0/1|2/0|0/2|1/1|2/2|2/1|1/2" $L] && ![regexp "1\\\|0|0\\\|1|2\\\|0|0\\\|2|1\\\|1|2\\\|2|2\\\|1|1\\\|2" $L]} {continue}

            set Ls [split $L "\t"]
            set chrom [lindex $Ls 0] 
            regsub -all "chr" $chrom "" chrom

            set pos [lindex $Ls 1]
            set ref [lindex $Ls 3]
            set alt [lindex $Ls 4] 

	    foreach altn [split $alt ","] {
		if {![regexp "^\[ACGTN*\]+$" $altn]} { 
		    puts "############################################################################"
		    puts "\t   WARNING: illegal character in VCF's ALT fields ('$altn')"
		    puts "\t   Please, check your $vcfFile file."
		    puts "\t   EXIT of VaRank!"
		    puts "############################################################################"
		    puts ""
		    exit
		}
		
		# If ALT=*, then variant is not wrote into the input file for annotation.
		if {$altn == "*"} {continue}

		## ID already annotated
		if {[lsearch -exact $lIDana "${chrom}_${pos}_${ref}_${altn}"] != -1} {continue}
           	set ID [AssignID $chrom $pos $ref $altn]
	    	if {[lsearch -exact $allIDfromVCFs $ID] == -1} {
		    lappend allIDfromVCFs $ID
		    ## Remove genotype like 1/2, 0/3, 0|2 (2 doesn't exist anymore)
		    ## All genotypes are set to 0/1
		    set Ln [concat $chrom [lrange $Ls 1 3] $altn "30\tPASS\tAC=1;DP=30\tGT:AD:GQ:DP\t0/1:20,10:40:30"]
		    set Ln [join $Ln "\t"]
		    lappend lVariantLines $Ln
		    incr nbL
		    if {$nbL > 40000} {
			set lVariantLines [lsort -command AscendingSortOnElement1 $lVariantLines]
			set lVariantLines [lsort -command AscendingSortOnElement0 $lVariantLines]
			WriteTextInFile [join $lVariantLines "\n"] $snpeffInputFile
			incr i
			while {[file exists $g_VaRank(vcfDir)/SnpEff/Output/allinone.$i.eff.varType.dbsnp.dbnsfp.phastCons.vcf]} {
			    incr i
			}
			set nbL 0
			set snpeffInputFile $g_VaRank(vcfDir)/SnpEff/Input/allinone.$i.vcf
			ReplaceTextInFile "#CHROM\tPOS\tID\tREF\tALT\tQUAL\tFILTER\tINFO\tFORMAT\ttutu" $snpeffInputFile			
			set lVariantLines {}
		    }
	    	} 
	    }
	}
    }
    set lVariantLines [lsort -command AscendingSortOnElement1 $lVariantLines]
    set lVariantLines [lsort -command AscendingSortOnElement0 $lVariantLines]

    if {[llength $lVariantLines] != 0} {
	WriteTextInFile [join $lVariantLines "\n"] $snpeffInputFile
    } else {
	file delete -force $snpeffInputFile
    }
}

proc ErrorWithinSnpEffFile {vcfOutputFile} {

    set test 1

    regexp ".(eff)(.varType)?(.dbsnp)?(.dbnsfp)?(.phastCons)?(.msigDb)?.vcf$" $vcfOutputFile match eff varType dbsnp dbnsfp phastCons msigDb
    
    set lTags {"##INFO=<ID=ANN"}
    if {$varType != ""} {lappend lTags "##INFO=<ID=VARTYPE"}
    if {$dbsnp != ""} {lappend lTags "##INFO=<ID=CAF"}
    if {$dbnsfp != ""} {lappend lTags "##INFO=<ID=dbNSFP"}
    if {$phastCons != ""} {lappend lTags "##INFO=<ID=PhastCons"}
    if {$msigDb != ""} {lappend lTags "##INFO=<ID=MSigDb"}
    
    # vcfOutputFile should exists
    if {![file exists $vcfOutputFile]} {return $test}

    # logfile should exists, even if empty
    if {![file exists $vcfOutputFile.log]} {return $test}

    # Check java logfile
    foreach L [LinesFromFile $vcfOutputFile.log] {
	if {[regexp "^Exception in thread|java.lang." $L]} {return $test}
    }

    # Check the vcf output file
    set lReads {}
    foreach L [LinesFromFile $vcfOutputFile] {
	if {![regexp "^#" $L]} {break}
	foreach tag $lTags {
	    if {[regexp "^$tag" $L]} {
		lappend lReads "$tag"
		break
	    }
	}
    }
    set lReads [lsort -unique $lReads]
    if {[llength $lReads] == [llength $lTags]} {set test 0}

    return $test
}


proc runOneSnpEffCommand {vcfOutputFile effCommand} {

    ReplaceTextInFile "SnpEff started: [clock format [clock seconds] -format "%B %d %Y - %H:%M"]\n" $vcfOutputFile.log
    WriteTextInFile "$effCommand\n" $vcfOutputFile.log

    if {[catch {eval exec $effCommand} Message]} {
	WriteTextInFile "$Message" $vcfOutputFile.log
	WriteTextInFile "SnpEff finished: [clock format [clock seconds] -format "%B %d %Y - %H:%M"]\n" $vcfOutputFile.log
	if {[ErrorWithinSnpEffFile $vcfOutputFile]} {
	    puts "############################################################################"
	    puts "\t   SnpEff error!"
	    puts "\t   Please, check the $vcfOutputFile.log file."
	    puts "\t   EXIT of VaRank!"
	    puts "############################################################################"
	    exit
	}
    }
}


proc checkSnpEff {} {

    global g_VaRank

    puts "...checking SnpEff ([clock format [clock seconds] -format "%B %d %Y - %H:%M"])"    

    # VCF input file for testing
    set vcfFile "$g_VaRank(SnpEffTestsDir)/example.vcf"

    if {![file exists $vcfFile]} {
	ReplaceTextInFile "#CHROM\tPOS\tID\tREF\tALT\tQUAL\tFILTER\tINFO\tFORMAT\tSample1" $vcfFile
	WriteTextInFile "1\t69511\t.\tA\tG\t136\tPASS\tAC=2;DP=11;FREQ_HTZ=1.14;FREQ_HOM=34.66;DB;MG;EVS\tGT:AD:GQ:DP\t1/1:0,11:30:11" $vcfFile
	WriteTextInFile "1\t762273\t.\tG\tA\t36\tPASS\tAC=1;DP=28;FREQ_HTZ=11.93;FREQ_HOM=26.7;DB;MG\tGT:AD:GQ:DP\t0/1:19,9:36:28" $vcfFile
    }

    # checking
    set effCommand "$g_VaRank(javaSnpEffCommand)/snpEff.jar"
    if {[catch {eval exec $effCommand} Message]} {
	if {[regexp "SnpEff version SnpEff (\[0-9\]+)\\.(\[0-9\]+)" $Message match version subversion]} {
	    if {$version < 4 || "$version.$subversion" == "4.0"} {
		puts "############################################################################"
		puts "\t   VaRank is compatible with SnpEff from version 4.1 "
		puts "\t   You are using version $version.$subversion ($g_VaRank(snpeffDir)/snpEff.jar)"
		puts "\t   Please install a new version of SnpEff"
		puts "\t   EXIT of VaRank!"
		puts "############################################################################"
		exit
	    }
	} else {
	    puts "############################################################################"
	    puts "\t   SnpEff error!"
	    puts "\t   $Message"
	    puts "\t   Please, check your SnpEff installation"
	    puts "\t   EXIT of VaRank!"
	    puts "############################################################################"
	    exit
	}
    } 
    puts "\t...using SnpEff version $version.$subversion"

    # If SnpEff checking has already been done previously, continue
    regsub ".vcf$" $vcfFile ".eff.varType.dbsnp.dbnsfp.phastCons.vcf" vcfOutputFile
    if {[file exists $vcfOutputFile] && ([file mtime $vcfOutputFile] > [file mtime $g_VaRank(snpeffDir)/snpEff.jar])} {
	return
    }

    # Remove outputfile if they already exist
    foreach outputFile [glob -nocomplain $g_VaRank(SnpEffTestsDir)/example.eff.*] {
	file delete -force $outputFile
    }

    puts "\t...checking step1: snpEff.jar eff annotation ([clock format [clock seconds] -format "%B %d %Y - %H:%M"])"
    regsub "vcf" $vcfFile "eff.vcf" vcfOutputFile
    set effCommand "$g_VaRank(javaSnpEffCommand)/snpEff.jar eff -c $g_VaRank(snpeffDir)/snpEff.config -v $g_VaRank(snpeffHumanDB) $vcfFile > $vcfOutputFile"
    runOneSnpEffCommand $vcfOutputFile $effCommand 
    set vcfFile $vcfOutputFile
    

    puts "\t...checking step2: SnpSift.jar varType annotation ([clock format [clock seconds] -format "%B %d %Y - %H:%M"])"
    regsub "vcf" $vcfFile "varType.vcf" vcfOutputFile
    set effCommand "$g_VaRank(javaSnpEffCommand)/SnpSift.jar varType $vcfFile > $vcfOutputFile"
    runOneSnpEffCommand $vcfOutputFile $effCommand 
    set vcfFile $vcfOutputFile

    puts "\t...checking step3: SnpSift.jar dbSNP annotation ([clock format [clock seconds] -format "%B %d %Y - %H:%M"])"
    regsub "vcf" $vcfFile "dbsnp.vcf" vcfOutputFile
    set effCommand "$g_VaRank(javaSnpEffCommand)/SnpSift.jar annotate $g_VaRank(dbSNP) $vcfFile > $vcfOutputFile"
    runOneSnpEffCommand $vcfOutputFile $effCommand 
    set vcfFile $vcfOutputFile

    puts "\t...checking step4: SnpSift.jar dbnsfp annotation ([clock format [clock seconds] -format "%B %d %Y - %H:%M"])"
    regsub "vcf" $vcfFile "dbnsfp.vcf" vcfOutputFile
    set effCommand "$g_VaRank(javaSnpEffCommand)/SnpSift.jar dbnsfp -db $g_VaRank(dbNSFP) $vcfFile > $vcfOutputFile"
    runOneSnpEffCommand $vcfOutputFile $effCommand 
    set vcfFile $vcfOutputFile

    puts "\t...checking step5: SnpSift.jar phastCons annotation ([clock format [clock seconds] -format "%B %d %Y - %H:%M"])"
    regsub "vcf" $vcfFile "phastCons.vcf" vcfOutputFile
    set effCommand "$g_VaRank(javaSnpEffCommand)/SnpSift.jar phastCons $g_VaRank(phastConsDB) $vcfFile > $vcfOutputFile"
    runOneSnpEffCommand $vcfOutputFile $effCommand 
    set vcfFile $vcfOutputFile

    #    puts "\t...checking step6: SnpSift.jar geneSets annotation ([clock format [clock seconds] -format "%B %d %Y - %H:%M"])"
    #    regsub "vcf" $vcfFile "msigDb.vcf" vcfOutputFile
    #    set effCommand "$g_VaRank(javaSnpEffCommand)/SnpSift.jar geneSets -v $g_VaRank(msigdb) $vcfFile > $vcfOutputFile"
    #    runOneSnpEffCommand $vcfOutputFile $effCommand 
    #    set vcfFile $vcfOutputFile


    file delete -force snpEff_genes.txt 
    file delete -force snpEff_summary.html 

    return
}



#################################################################################################
# OUTPUTS:
#    - $g_VaRank(vcfDir)/SnpEff/'vcfFile'.eff.varType.dbsnp.dbnsfp.phastCons.vcf
#    (- $g_VaRank(vcfDir)/SnpEff/'vcfFile'.eff.varType.dbsnp.dbnsfp.phastCons.msigDb.vcf)
#
# RETURN :
#    - return "1" if SnpEff has been run for all the variations
#    - else "exit" 
#################################################################################################
proc runSnpEff {} {

    global g_VaRank

    ## Running SnpEff 
    #
    puts "...running SnpEff ([clock format [clock seconds] -format "%B %d %Y - %H:%M"])"
    file mkdir $g_VaRank(vcfDir)/SnpEff/Output
    foreach vcfFile [glob -nocomplain $g_VaRank(vcfDir)/SnpEff/Input/*.vcf $g_VaRank(vcfDir)/SnpEff/Input/*.vcf.gz] {
	puts "\t...Annotation of $vcfFile"
	
	## Checking if all annotations are already done 
	#regsub "vcf(.gz)?" $vcfFile "eff.varType.dbsnp.dbnsfp.phastCons.msigDb.vcf" vcfOutputFile
	regsub "vcf(.gz)?" $vcfFile "eff.varType.dbsnp.dbnsfp.phastCons.vcf" vcfOutputFile	
	regsub "/Input/" $vcfOutputFile "/Output/" vcfOutputFile

	if {[ErrorWithinSnpEffFile $vcfOutputFile]} {

	    puts "\tstep1: snpEff.jar eff annotation ([clock format [clock seconds] -format "%B %d %Y - %H:%M"])"
	    regsub "vcf(.gz)?" $vcfFile "eff.vcf" vcfOutputFile
	    regsub "/Input/" $vcfOutputFile "/Output/" vcfOutputFile
	    set effCommand "$g_VaRank(javaSnpEffCommand)/snpEff.jar eff -c $g_VaRank(snpeffDir)/snpEff.config -v $g_VaRank(snpeffHumanDB) $vcfFile > $vcfOutputFile"
	    runOneSnpEffCommand $vcfOutputFile $effCommand 
	    set vcfFile $vcfOutputFile

	    puts "\tstep2: SnpSift.jar varType annotation ([clock format [clock seconds] -format "%B %d %Y - %H:%M"])"
	    regsub "vcf" $vcfFile "varType.vcf" vcfOutputFile
	    set effCommand "$g_VaRank(javaSnpEffCommand)/SnpSift.jar varType $vcfFile > $vcfOutputFile"
	    runOneSnpEffCommand $vcfOutputFile $effCommand 
	    file delete -force $vcfFile 
	    set vcfFile $vcfOutputFile

	    puts "\tstep3: SnpSift.jar dbSNP annotation ([clock format [clock seconds] -format "%B %d %Y - %H:%M"])"
	    regsub "vcf" $vcfFile "dbsnp.vcf" vcfOutputFile
	    set effCommand "$g_VaRank(javaSnpEffCommand)/SnpSift.jar annotate $g_VaRank(dbSNP) $vcfFile > $vcfOutputFile"
	    runOneSnpEffCommand $vcfOutputFile $effCommand 
	    file delete -force $vcfFile 
	    set vcfFile $vcfOutputFile

	    puts "\tstep4: SnpSift.jar dbnsfp annotation ([clock format [clock seconds] -format "%B %d %Y - %H:%M"])"
	    regsub "vcf" $vcfFile "dbnsfp.vcf" vcfOutputFile
	    set effCommand "$g_VaRank(javaSnpEffCommand)/SnpSift.jar dbnsfp -db $g_VaRank(dbNSFP) $vcfFile > $vcfOutputFile"
	    runOneSnpEffCommand $vcfOutputFile $effCommand 
	    file delete -force $vcfFile 
	    set vcfFile $vcfOutputFile

	    puts "\tstep5: SnpSift.jar phastCons annotation ([clock format [clock seconds] -format "%B %d %Y - %H:%M"])"
	    regsub "vcf" $vcfFile "phastCons.vcf" vcfOutputFile
	    set effCommand "$g_VaRank(javaSnpEffCommand)/SnpSift.jar phastCons $g_VaRank(phastConsDB) $vcfFile > $vcfOutputFile"
	    runOneSnpEffCommand $vcfOutputFile $effCommand 
	    file delete -force $vcfFile 
	    set vcfFile $vcfOutputFile

	    #	    puts "\tstep6: SnpSift.jar geneSets annotation ([clock format [clock seconds] -format "%B %d %Y - %H:%M"])"
	    #	    regsub "vcf" $vcfFile "msigDb.vcf" vcfOutputFile
	    #	    set effCommand "$g_VaRank(javaSnpEffCommand)/SnpSift.jar geneSets -v $g_VaRank(msigdb) $vcfFile > $vcfOutputFile"
	    #	    runOneSnpEffCommand $vcfOutputFile $effCommand 
	    #	    file delete -force $vcfFile 
	    #	    set vcfFile $vcfOutputFile

	    file delete -force snpEff_genes.txt
	    file delete -force snpEff_summary.html

	} else {
	    puts "\t   already done."
	}
    }

    return 1
}

proc parseSnpEffFile {{afterPPH 0}} {

    global g_ANNOTATION
    global g_VaRank
    global g_vcfINFOS
    global g_vcfINFOS_SnpEff

    ## Determine the header of g_ANNOTATION() (= g_ANNOTATION(#id))
    ###############################################################
    set header {gene omimId transcript transLen Uniprot protein posAA wtAA_1 varAA_1 varType codingEffect varLocation exon intron gNomen cNomen pNomen hgmdId hgmdPhenotype hgmdPubMedId rsId clinVarReviewStatus rsClinicalSignificance clinVarClinSignifs rsAncestralAllele rsHeterozygosity rsMAF rsMAFAllele rsMAFCount 1000g_AF 1000g_AFR_AF 1000g_SAS_AF 1000g_EAS_AF 1000g_EUR_AF 1000g_AMR_AF espRefEACount espRefAACount espRefAllCount espAltEACount espAltAACount espAltAllCount espEAMAF espAAMAF espAllMAF espAvgReadDepth wtMaxEntScore varMaxEntScore wtSSFScore varSSFScore wtNNSScore varNNSScore distNearestSS nearestSSType localSpliceEffect SIFTprediction SIFTweight SIFTmedian PPH2class phyloP phastCons granthamDist clinVarIds clinVarOrigins clinVarMethods clinVarPhenotypes exacAFRFreq exacSASFreq exacEASFreq exacAMRFreq exacNFEFreq exacFINFreq exacOTHFreq exacAllFreq exacAFRHmz exacSASHmz exacEASHmz exacEASHmz exacAMRHmz exacNFEHmz exacFINHmz exacOTHHmz exacFilter exacReadDepth}
    set g_ANNOTATION(#id) [join $header "\t"]

    
    ####################################################################################
    #################### Description of the VCF INFO/FORMAT fields #####################
    ####################################################################################
    ##INFO=<ID=AC,Number=.,Type=Integer,Description="Allele count in genotypes, for each ALT allele, in the same order as listed">
    ##INFO=<ID=DP,Number=1,Type=Integer,Description="Total Depth accross all samples">
    ##INFO=<ID=FREQ_HTZ,Number=1,Type=Float,Description="Integragen's Heterozygote Frequency">
    ##INFO=<ID=FREQ_HOM,Number=1,Type=Float,Description="Integragen's Homozygote Frequency">

    ##FORMAT=<ID=GT,Number=1,Type=String,Description="Genotype">
    ##FORMAT=<ID=GQ,Number=1,Type=Integer,Description="Genotype Quality">
    ##FORMAT=<ID=DP,Number=1,Type=Integer,Description="Read Depth for sample">
    ##FORMAT=<ID=AD,Number=1,Type=Integer,Description="Allelic depths for the REF and ALT alleles in order listed">

    #####################################################################################
    ##  SnpEffCmd="SnpEff  GRCh37.75 ./Sample1.vcf "
    #####################################################################################
    ##INFO=<ID=ANN,Number=.,Type=String,Description="Functional annotations: 'Allele | Annotation | Annotation_Impact | Gene_Name | Gene_ID | Feature_Type | Feature_ID | Transcript_BioType | Rank | HGVS.c | HGVS.p | cDNA.pos / cDNA.length | CDS.pos / CDS.length | AA.pos / AA.length | Distance | ERRORS / WARNINGS / INFO' ">
    ##INFO=<ID=LOF,Number=.,Type=String,Description="Predicted loss of function effects for this variant. Format: 'Gene_Name | Gene_ID | Number_of_transcripts_in_gene | Percent_of_transcripts_affected' ">
    ##INFO=<ID=NMD,Number=.,Type=String,Description="Predicted nonsense mediated decay effects for this variant. Format: 'Gene_Name | Gene_ID | Number_of_transcripts_in_gene | Percent_of_transcripts_affected' ">

    #####################################################################################
    ##  SnpSiftCmd="SnpSift varType ./SnpEff/Sample1.eff.vcf"
    #####################################################################################
    ##INFO=<ID=VARTYPE,Number=A,Type=Flag,Description="Variant types {SNP,MNP,INS,DEL,Mixed}">
    ##INFO=<ID=VARTYPE,Number=A,Type=String,Description="Comma separated list of variant types. One per allele">

    #####################################################################################
    ##  SnpSiftCmd="SnpSift annotate ./SnpEff/db/dbSNP.2015-01-09_00-All.vcf ./SnpEff/Sample1.eff.varType.vcf"
    #####################################################################################
    ##INFO=<ID=DSS,Number=0,Type=Flag,Description="In donor splice-site FxnCode = 75">
    ##INFO=<ID=OM,Number=0,Type=Flag,Description="Has OMIM/OMIA">
    ##INFO=<ID=MUT,Number=0,Type=Flag,Description="Is mutation (journal citation, explicit fact): a low frequency variation that is cited in journal and other reputable sources">
    ##INFO=<ID=KGPhase1,Number=0,Type=Flag,Description="1000 Genome phase 1 (incl. June Interim phase 1)">
    ##INFO=<ID=KGPhase3,Number=0,Type=Flag,Description="1000 Genome phase 3">
    ##INFO=<ID=ASS,Number=0,Type=Flag,Description="In acceptor splice site FxnCode = 73">
    ##INFO=<ID=CAF,Number=.,Type=String,Description="An ordered, comma delimited list of allele frequencies based on 1000Genomes, starting with the reference allele followed by alternate alleles as ordered in the ALT column. Where a 1000Genomes alternate allele is not in the dbSNPs alternate allele set, the allele is added to the ALT column.  The minor allele is the second largest value in the list, and was previuosly reported in VCF as the GMAF.  This is the GMAF reported on the RefSNP and EntrezSNP pages and VariationReporter">
    ##INFO=<ID=S3D,Number=0,Type=Flag,Description="Has 3D structure - SNP3D table">
    ##INFO=<ID=R5,Number=0,Type=Flag,Description="In 5' gene region FxnCode = 15">
    ##INFO=<ID=R3,Number=0,Type=Flag,Description="In 3' gene region FxnCode = 13">
    ##INFO=<ID=RS,Number=1,Type=Integer,Description="dbSNP ID (i.e. rs number)">

    #####################################################################################
    ##  SnpSiftCmd="SnpSift dbnsfp ./SnpEff/Sample1.eff.varType.dbsnp.vcf"
    #####################################################################################
    ##INFO=<ID=dbNSFP_GERP++_RS,Number=A,Type=Float,Description="Field 'GERP++_RS' from dbNSFP">
    ##INFO=<ID=dbNSFP_1000Gp1_ASN_AF,Number=A,Type=Float,Description="Field '1000Gp1_ASN_AF' from dbNSFP">
    ##INFO=<ID=dbNSFP_Uniprot_acc,Number=A,Type=String,Description="Field 'Uniprot_acc' from dbNSFP">
    ##INFO=<ID=dbNSFP_MutationTaster_pred,Number=A,Type=Character,Description="Field 'MutationTaster_pred' from dbNSFP">
    ##INFO=<ID=dbNSFP_ESP6500_AA_AF,Number=A,Type=Float,Description="Field 'ESP6500_AA_AF' from dbNSFP">
    ##INFO=<ID=dbNSFP_SIFT_pred,Number=A,Type=Character,Description="Field 'SIFT_pred' from dbNSFP">
    ##INFO=<ID=dbNSFP_phastCons100way_vertebrate,Number=A,Type=Float,Description="Field 'phastCons100way_vertebrate' from dbNSFP">
    ##INFO=<ID=dbNSFP_Polyphen2_HDIV_pred,Number=A,Type=Character,Description="Field 'Polyphen2_HDIV_pred' from dbNSFP">
    ##INFO=<ID=dbNSFP_1000Gp1_EUR_AF,Number=A,Type=Float,Description="Field '1000Gp1_EUR_AF' from dbNSFP">
    ##INFO=<ID=dbNSFP_1000Gp1_AFR_AF,Number=A,Type=Float,Description="Field '1000Gp1_AFR_AF' from dbNSFP">
    ##INFO=<ID=dbNSFP_ESP6500_EA_AF,Number=A,Type=Float,Description="Field 'ESP6500_EA_AF' from dbNSFP">
    ##INFO=<ID=dbNSFP_LRT_pred,Number=A,Type=Character,Description="Field 'LRT_pred' from dbNSFP">
    ##INFO=<ID=dbNSFP_1000Gp1_AMR_AF,Number=A,Type=Float,Description="Field '1000Gp1_AMR_AF' from dbNSFP">
    ##INFO=<ID=dbNSFP_GERP++_NR,Number=A,Type=Float,Description="Field 'GERP++_NR' from dbNSFP">
    ##INFO=<ID=dbNSFP_1000Gp1_AF,Number=A,Type=Float,Description="Field '1000Gp1_AF' from dbNSFP">
    ##INFO=<ID=dbNSFP_Polyphen2_HVAR_pred,Number=A,Type=Character,Description="Field 'Polyphen2_HVAR_pred' from dbNSFP">
    ##INFO=<ID=dbNSFP_Interpro_domain,Number=A,Type=String,Description="Field 'Interpro_domain' from dbNSFP">

    #####################################################################################
    ##  SnpSiftCmd="SnpSift phastCons ./SnpEff/db/phastCons/ ./SnpEff/Sample1.eff.varType.dbsnp.dbnsfp.vcf"
    #####################################################################################
    ##INFO=<ID=PhastCons,Number=1,Type=Float,Description="PhastCons conservation score">

    #####################################################################################
    ##  SnpSiftCmd="SnpSift geneSets ./SnpEff/db/msigDb/msigdb.v4.0.symbols.gmt ./SnpEff/Sample1.eff.varType.dbsnp.dbnsfp.phastCons.vcf"
    #####################################################################################
    ##INFO=<ID=MSigDb,Number=.,Type=String,Description="Gene set from MSigDB database (GSEA)">


    #####################################################################################
    ## VCF header line: 
    #####################################################################################
    #CHROM\tPOS\tID\tREF\tALT\tQUAL\tFILTER\tINFO\tFORMAT\tSample1


    #####################################################################################
    ## VCF annotation line example:
    #####################################################################################
    ##1       69511   rs2691305;rs75062661    A       G       136.0   PASS    AC=2;DP=11;FREQ_HTZ=1.14;FREQ_HOM=34.66;DB;MG;EVS;ANN=G|missensevariant|MODERATE|OR4F5|ENSG00000186092|transcript|ENST00000335137|protein_coding|1/1|c.421A>G|p.Thr141Ala|421/918|421/918|141/305||,G|sequence_feature|LOW|OR4F5|ENSG00000186092|disulfide_bond|ENST00000335137|protein_coding|1/1|c.421A>G||||||,G|sequence_feature|LOW|OR4F5|ENSG00000186092|transmembrane_region:Transmembrane_region|ENST00000335137|protein_coding|1/1|c.421A>G||||||;SNP;HOM;VARTYPE=SNP;ASP=true;GNO=true;KGPhase1=true;NSM=true;REF=true;RS=75062661;RSPOS=69511;S3D=true;SAO=0;SSR=0;VC=SNV;VLD=true;VP=0x050200000a05040116000100;WGT=1;dbSNPBuildID=131;dbNSFP_GERP++_RS=1.15;dbNSFP_1000Gp1_ASN_AF=0.8741258741258742;dbNSFP_Uniprot_acc=Q8NH21;dbNSFP_MutationTaster_pred=P;dbNSFP_ESP6500_AA_AF=0.544101;dbNSFP_SIFT_pred=T;dbNSFP_phastCons100way_vertebrate=0.000000;dbNSFP_Polyphen2_HDIV_pred=B;dbNSFP_1000Gp1_EUR_AF=0.6952506596306068;dbNSFP_1000Gp1_AFR_AF=0.32926829268292684;dbNSFP_ESP6500_EA_AF=0.887429;dbNSFP_LRT_pred=N;dbNSFP_1000Gp1_AMR_AF=0.649171270718232;dbNSFP_GERP++_NR=2.31;dbNSFP_1000Gp1_AF=0.652014652014652;dbNSFP_Polyphen2_HVAR_pred=B;dbNSFP_Interpro_domain=GPCR,_rhodopsin-like_superfamily_(1),;MSigDb=ACEVEDO_METHYLATED_IN_LIVER_CANCER_DN,KEGG_OLFACTORY_TRANSDUCTION,REACTOME_GPCR_DOWNSTREAM_SIGNALING,REACTOME_OLFACTORY_SIGNALING_PATHWAY,REACTOME_SIGNALING_BY_GPCR,chr1p36     GT:AD:GQ:DP     1/1:0,11:30:11_


    #####################################################################################
    ## VCF INFO annotation line example:
    #####################################################################################
    ## ANN=G|missensevariant|MODERATE|OR4F5|ENSG00000186092|transcript|ENST00000335137|protein_coding|1/1|c.421A>G|p.Thr141Ala|421/918|421/918|141/305||,G|sequence_feature|LOW|OR4F5|ENSG00000186092|disulfide_bond|ENST00000335137|protein_coding|1/1|c.421A>G||||||,G|sequence_feature|LOW|OR4F5|ENSG00000186092|transmembrane_region:Transmembrane_region|ENST00000335137|protein_coding|1/1|c.421A>G||||||;
    ## VARTYPE=SNP;
    ## RS=75062661;
    ## dbNSFP_Uniprot_acc=Q8NH21;
    ## dbNSFP_MutationTaster_pred=P;
    ## dbNSFP_SIFT_pred=T;
    ## dbNSFP_Polyphen2_HVAR_pred=B;
    ## dbNSFP_phastCons100way_vertebrate=0.000000;
    ## dbNSFP_1000Gp1_ASN_AF=0.8741258741258742;
    ## dbNSFP_1000Gp1_EUR_AF=0.6952506596306068;
    ## dbNSFP_1000Gp1_AFR_AF=0.32926829268292684;
    ## dbNSFP_1000Gp1_AMR_AF=0.649171270718232;
    ## dbNSFP_1000Gp1_AF=0.652014652014652;
    ## dbNSFP_ESP6500_AA_AF=0.544101;
    ## dbNSFP_ESP6500_EA_AF=0.887429;
    ## dbNSFP_Interpro_domain=GPCR,_rhodopsin-like_superfamily_(1),;
    ## MSigDb=ACEVEDO_METHYLATED_IN_LIVER_CANCER_DN,KEGG_OLFACTORY_TRANSDUCTION,REACTOME_GPCR_DOWNSTREAM_SIGNALING,REACTOME_OLFACTORY_SIGNALING_PATHWAY,REACTOME_SIGNALING_BY_GPCR,chr1p36

    puts "...parsing SnpEff annotations ([clock format [clock seconds] -format "%B %d %Y - %H:%M"])"
    #foreach annFile [glob -nocomplain $g_VaRank(vcfDir)/SnpEff/*.eff.varType.dbsnp.dbnsfp.phastCons.msigDb.vcf] 
    foreach annFile [glob -nocomplain $g_VaRank(vcfDir)/SnpEff/Output/*.eff.varType.dbsnp.dbnsfp.phastCons.vcf] {
	puts "\t$annFile"
	foreach L [LinesFromFile $annFile] {
#	    if {[regexp "^##INFO=<ID=ANN.+: '(\[^'\]+)'" $L match lAnn]} { ; #doesn't work
#		regsub -all " " $lAnn "" lAnn
#		regsub "ERRORS/WARNINGS/INFO$" $lAnn "" lAnn
#		set lVariables {}
#		set expression "ANN="
#		while {[regexp "^(\[^|/\]+)(\[|/\])" $lAnn match var signe]} {
#		    lappend lVariables $var
#		    if {$signe == "|"} {
#			append expression "(\\\[^|/\\\]*)\\\\|"
#		    } else {
#			append expression "(\\\[^|/\\\]*)/"
#		    }
#		    regsub "^(\[^|/\]+)(\[|/\])" $lAnn "" lAnn
#		}
#		lappend lVariables {*}"ERRORS WARNINGS INFOsup"
#		append expression "(\\\[^|,/\\\]*)/?(\\\[^|,/\\\]*)/?(\\\[^|/,\\\]*)"
#		continue
#	    }
	    if {[regexp "^##" $L]} {continue}
	    if {[regexp "^#CHROM" $L]} {
		set Ls [split $L "\t"]
		set i_chr    [lsearch -exact $Ls "#CHROM" ]; if {$i_chr    == -1} {puts "Bad header line syntax into $vcfFile. \n#CHROM column not found - Exit"; exit}
		set i_pos    [lsearch -exact $Ls "POS"];     if {$i_pos    == -1} {puts "Bad header line syntax into $vcfFile. \nPOS column not found - Exit"; exit}
		set i_ref    [lsearch -exact $Ls "REF"];     if {$i_ref    == -1} {puts "Bad header line syntax into $vcfFile. \nREF column not found - Exit"; exit}
		set i_alt    [lsearch -exact $Ls "ALT"];     if {$i_alt    == -1} {puts "Bad header line syntax into $vcfFile. \nALT column not found - Exit"; exit}
		set i_id     [lsearch -exact $Ls "ID"];      if {$i_id     == -1} {puts "Bad header line syntax into $vcfFile. \nID column not found - Exit"; exit}
		set i_info   [lsearch -exact $Ls "INFO"];    if {$i_info   == -1} {puts "Bad header line syntax into $vcfFile. \nINFO column not found - Exit"; exit}
		continue
	    }

	    if {$L == ""} {continue}

	    ## If there is only normal hom in the line (GT = 0|0 or 0/0), so there isn't mutation to analyse by VaRank
	    if {![regexp "1/0|0/1|2/0|0/2|1/1|2/2|2/1|1/2" $L] && ![regexp "1\\\|0|0\\\|1|2\\\|0|0\\\|2|1\\\|1|2\\\|2|2\\\|1|1\\\|2" $L]} {continue}

	    set Ls [split $L "\t"]
	    set chrom [lindex $Ls $i_chr] 
	    regsub -all "^chr" $chrom "" chrom

	    set pos [lindex $Ls $i_pos] 
	    set ref [lindex $Ls $i_ref] 
	    set alt [lindex $Ls $i_alt] 
	    set rs [lindex $Ls $i_id] 
	    if {[isNotAnRS $rs]} {
		set rs "NA"
	    } 

	    ## ALT=T --> ANN=T|...
	    ## ALT=G,T --> ANN=G|... , T|...
	    ## ALT=C,G (cancer) --> ANN=G-C|... (format = ALT-REFERENCE)
	    ## ALT=G (into MNP)--> ANN=G-chr1:123456_A>T|...

	    ## Multiple alt (G,T) have been split before SnpEff annotation: 1 alt per line in annotated VCF
	    set INFO [lindex $Ls $i_info] 
	    regsub -all {\"} $INFO "" INFO

	    set VARTYPE "NA"; 			regexp "VARTYPE=(\[^;\]+);" $INFO match VARTYPE
	    set LOF "NA"; 			regexp "LOF=(\[^;\]+);" $INFO match LOF
	    set NMD "NA";         		regexp "NMD=(\[^;\]+);" $INFO match NMD
	    set EVS "NA";         		regexp "EVS=(\[^;\]+);" $INFO match EVS 
	    set AC "NA";       			regexp "AC=(\[^;\]+);" $INFO match AC
	    set DP "NA";         		regexp "DP=(\[^;\]+);" $INFO match DP
	    set FREQ_HTZ "NA";    		regexp "FREQ_HTZ=(\[^;\]+);" $INFO match FREQ_HTZ
	    set FREQ_HOM "NA";    		regexp "FREQ_HOM=(\[^;\]+);" $INFO match FREQ_HOM
	    set COMMON "NA";      		regexp "COMMON=(\[^;\]+);" $INFO match COMMON
	    set MUT "NA";         		regexp "MUT=(\[^;\]+);" $INFO match MUT
	    set CAF "NA";         		regexp "CAF=(\[^;\]+);" $INFO match CAF
	    set dbNSFP_1000Gp1_ASN_AF "NA";   	regexp "dbNSFP_1000Gp1_ASN_AF=(\[^;\]+);" $INFO match dbNSFP_1000Gp1_ASN_AF
	    set dbNSFP_Uniprot_acc "NA";        regexp "dbNSFP_Uniprot_acc=(\[^;\]+);" $INFO match dbNSFP_Uniprot_acc
	    set dbNSFP_ESP6500_AA_AF "NA";      regexp "dbNSFP_ESP6500_AA_AF=(\[^;\]+);" $INFO match dbNSFP_ESP6500_AA_AF
	    set dbNSFP_SIFT_pred "NA";         	regexp "dbNSFP_SIFT_pred=(\[^;\]+);" $INFO match dbNSFP_SIFT_pred
	    set dbNSFP_phastCons "NA";         	regexp "dbNSFP_phastCons100way_vertebrate=(\[^;\]+);" $INFO match dbNSFP_phastCons
	    set dbNSFP_1000Gp1_EUR_AF "NA";     regexp "dbNSFP_1000Gp1_EUR_AF=(\[^;\]+);" $INFO match dbNSFP_1000Gp1_EUR_AF
	    set dbNSFP_1000Gp1_AFR_AF "NA";     regexp "dbNSFP_1000Gp1_AFR_AF=(\[^;\]+);" $INFO match dbNSFP_1000Gp1_AFR_AF
	    set dbNSFP_ESP6500_EA_AF "NA";      regexp "dbNSFP_ESP6500_EA_AF=(\[^;\]+);" $INFO match dbNSFP_ESP6500_EA_AF
	    set dbNSFP_1000Gp1_AMR_AF "NA";     regexp "dbNSFP_1000Gp1_AMR_AF=(\[^;\]+);" $INFO match dbNSFP_1000Gp1_AMR_AF
	    set dbNSFP_1000Gp1_AF "NA";     	regexp "dbNSFP_1000Gp1_AF=(\[^;\]+);" $INFO match dbNSFP_1000Gp1_AF
	    set dbNSFP_Polyphen2_HVAR_pred "NA";regexp "dbNSFP_Polyphen2_HVAR_pred=(\[^;\]+);" $INFO match dbNSFP_Polyphen2_HVAR_pred
	    set dbNSFP_Interpro_domain "NA";    regexp "dbNSFP_Interpro_domain=(\[^;\]+);" $INFO match dbNSFP_Interpro_domain
	    set MSigDb "NA";    		regexp "MSigDb=(\[^;\]+);" $INFO match MSigDb

	    foreach dbNSFPval {FREQ_HTZ FREQ_HOM dbNSFP_1000Gp1_ASN_AF dbNSFP_Uniprot_acc dbNSFP_ESP6500_AA_AF dbNSFP_SIFT_pred dbNSFP_phastCons dbNSFP_1000Gp1_EUR_AF dbNSFP_1000Gp1_AFR_AF dbNSFP_ESP6500_EA_AF dbNSFP_1000Gp1_AMR_AF dbNSFP_1000Gp1_AF dbNSFP_Polyphen2_HVAR_pred} {
		foreach el [split [set $dbNSFPval] ","] {
		    if {$el == "" || $el == "."} {
			set [set dbNSFPval] "NA"
		    } else {
			set [set dbNSFPval] $el
			break
		    }
		}
		
	    }

	    ### In case of several prediction (T,D,D), keep the deleterious effect
            if {[regexp "T" $dbNSFP_SIFT_pred]} {set dbNSFP_SIFT_pred "Tolerated"}
	    if {[regexp "D" $dbNSFP_SIFT_pred]} {set dbNSFP_SIFT_pred "Deleterious"}

	    ## If $PPH environment variable is not defined
	    if {$g_VaRank(pph2Dir) == ""} {
		## D=probably damaging, P=possibly damaging, B=benign
		if {[regexp "^D" $dbNSFP_Polyphen2_HVAR_pred]} {set dbNSFP_Polyphen2_HVAR_pred "probably damaging"}
		if {[regexp "^P" $dbNSFP_Polyphen2_HVAR_pred]} {set dbNSFP_Polyphen2_HVAR_pred "possibly damaging"}
		if {[regexp "^B" $dbNSFP_Polyphen2_HVAR_pred]} {set dbNSFP_Polyphen2_HVAR_pred "neutral"}
	    } else {
		## $PPH environment variable is defined: don't use dbNSFP_Polyphen2_HVAR_pred
		set dbNSFP_Polyphen2_HVAR_pred "NA"
	    }


	    ## Listing of all genes at this position (e.g. gene="MROHS/FLJ43860")
	    set lINFO [split $INFO ";"]
	    set i_ann [lsearch -regexp $lINFO "^ANN="]
	    set lAnn [lindex $lINFO $i_ann]
	    regsub "^ANN=" $lAnn "" lAnn	
	    set lAnn [split $lAnn ","]
	    set lGenes {}
	    foreach ann $lAnn {
		set ann [split $ann "|"]
		lappend lGenes [lindex $ann 3] ; ### Change the code here!!!! Ne pas coder 3 en dur.
	    }
	    set firstGN [lindex $lGenes 0]
	    set lGenes [lsort -unique $lGenes]
	    # Keep the first annotated gene (the “most deleterious”) in first position
	    set gene "$firstGN"
	    foreach g $lGenes {
		if {$g == ""} {continue}
		if {$g != $firstGN} {lappend gene $g}
	    }
	    set gene [join $gene "/"]


	    ## ALT=TAA --> ANN=TAA|...
	    ## ALT=G,T --> ANN=G|... , T|...
	    ## ALT=C,G (cancer) --> ANN=G-C|... (format = ALT-REFERENCE)
	    ## ALT=G (into MNP)--> ANN=G-chr1:123456_A>T|...	    	

	    # Keep annotations of the first annotated variant (the “most deleterious”) (only if $Annotation is not equal to "sequence_feature")
	    foreach ann $lAnn {
		if {[regexp "(\[^|\]*)\\|(\[^|\]*)\\|(\[^|\]*)\\|(\[^|\]*)\\|(\[^|\]*)\\|(\[^|\]*)\\|(\[^|\]*)\\|(\[^|\]*)\\|(\[^|\]*)\\|(\[^|\]*)\\|(\[^|\]*)\\|(\[^|/\]*)/?(\[^|/\]*)\\|(\[^|/\]*)/?(\[^|/\]*)\\|(\[^|/\]*)/?(\[^|/\]*)\\|(\[^|\]*)\\|(\[^|,/\]*)/?(\[^|,/\]*)/?(\[^|/,;\]*)" $ann match Allele Annotation Annotation_Impact Gene_Name Gene_ID Feature_Type Feature_ID Transcript_BioType Rank HGVS.c HGVS.p cDNA.pos cDNA.length CDS.pos CDS.length AA.pos AA.length Distance ERRORS WARNINGS INFOsup]} {
		    #if {![eval regexp "$expression" $INFO match $lVariables]} {} ; #doesn't work

		    foreach val {Allele Annotation Annotation_Impact Gene_Name Gene_ID Feature_Type Feature_ID Transcript_BioType Rank HGVS.c HGVS.p cDNA.pos cDNA.length CDS.pos CDS.length AA.pos AA.length Distance ERRORS WARNINGS INFOsup} {
			if {[set $val] == ""} {set $val "NA"}
		    }

		    if {[info exists g_VaRank(DEBUG)]} {
			puts "INFO: $INFO"
			puts "Allele: $Allele"
			puts "Annotation: $Annotation"
			puts "Annotation_Impact: $Annotation_Impact"
			puts "Gene_Name: $Gene_Name"
			puts "Gene_ID: $Gene_ID"
			puts "Feature_Type: $Feature_Type"
			puts "Feature_ID: $Feature_ID"
			puts "Transcript_BioType: $Transcript_BioType"
			puts "Rank: $Rank"
			puts "HGVS.c: ${HGVS.c}"
			puts "HGVS.p: ${HGVS.p}"
			puts "cDNA.pos: ${cDNA.pos}"
			puts "cDNA.length: ${cDNA.length}"
			puts "CDS.pos: ${CDS.pos}"
			puts "CDS.length: ${CDS.length}"
			puts "AA.pos: ${AA.pos}"
			puts "AA.length: ${AA.length}"
			puts "Distance: $Distance"
			puts "ERRORS: $ERRORS"
			puts "WARNINGS: $WARNINGS"
			puts "INFOsup: $INFOsup"
		    }
		    
		    set wtAA_1 "NA"; set posAA "NA"; set varAA_1 "NA"
		    ## p.His309fs*  p.Phe508del p.His7dup p.Trp33_Lys35delinsArg p.Met1ext-5
		    if {[regexp "p\\.(\[A-Za-z\]{3})(\[0-9\]+)(\[A-Za-z*\]+)" ${HGVS.p} match wtAA_1 posAA varAA_1]} {
			if {[regexp "fs" $varAA_1]} {set varAA_1 "*"}
			set wtAA_1 [correspondanceAA3inAA1 $wtAA_1]
			set varAA_1 [correspondanceAA3inAA1 $varAA_1]
		    }

		    ## set header {gene omimId transcript transLen Uniprot protein posAA wtAA_1 varAA_1 varType codingEffect varLocation exon intron gNomen cNomen pNomen hgmdId hgmdPhenotype hgmdPubMedId rsId clinVarReviewStatus rsClinicalSignificance clinVarClinSignifs rsAncestralAllele rsHeterozygosity rsMAF rsMAFAllele rsMAFCount 1000g_AF 1000g_AFR_AF 1000g_SAS_AF 1000g_EAS_AF 1000g_EUR_AF 1000g_AMR_AF espRefEACount espRefAACount espRefAllCount espAltEACount espAltAACount espAltAllCount espEAMAF espAAMAF espAllMAF espAvgReadDepth wtMaxEntScore varMaxEntScore wtSSFScore varSSFScore wtNNSScore varNNSScore distNearestSS nearestSSType localSpliceEffect SIFTprediction SIFTweight SIFTmedian PPH2class phyloP phastCons granthamDist}

		    ## Effect into Sequence Ontology (codingEffect)
		    ## Exon: exon_variant exon_loss_variant frameshift_variant missense_variant initiator_codon_variant stop_retained_variant rare_amino_acid_variant stop_lost start_lost stop_gained synonymous_variant start_retained stop_retained_variant 
		    ## Intron: intron_variant conserved_intron_variant splice_acceptor_variant splice_donnor_variant
		    ## Exon/Intron: splice_region_variant

		    set varLocation "NA"
		    set exon "NA"
		    set intron "NA"
		    if {[regexp "exon_variant|exon_loss_variant|frameshift_variant|missense_variant|initiator_codon_variant|stop_retained_variant|rare_amino_acid_variant|stop_lost|start_lost|stop_gained|synonymous_variant|start_retained|stop_retained_variant" $Annotation]} {
			set varLocation "exon"
			set exon '$Rank'
		    } elseif {[regexp "intron_variant|conserved_intron_variant|splice_acceptor_variant|splice_donnor_variant" $Annotation]} {
			set varLocation "intron"
			set intron '$Rank'
		    } elseif {[regexp "5_prime" $Annotation]} {
			set varLocation "5'UTR"
		    } elseif {[regexp "3_prime" $Annotation]} {
			set varLocation "3'UTR"
		    } elseif {[regexp "downstream" $Annotation]} {
			set varLocation "downstream"
		    } elseif {[regexp "upstream" $Annotation]} {
			set varLocation "upstream"
		    }
		    
		    ## Annotation == frameshift_variant&missense_variant
		    ## keep only the first annotation (important for statistics)
		    set Annotation [lindex [split $Annotation "&"] 0]
		    lappend g_VaRank(Annotation) $Annotation


		    #########################################
		    ## Determine/simplify the ID
		    #########################################
		    set ID [AssignID $chrom $pos $ref $alt]		    


		    ## Don't put a "set" in place of the "lappend"! (else bug during the scoring)
		    if {[info exists g_ANNOTATION($ID)]} {unset g_ANNOTATION($ID)}
		    lappend g_ANNOTATION($ID) "$gene\tNA\tNA\tNA\t$dbNSFP_Uniprot_acc\tNA\t${AA.pos}\t$wtAA_1\t$varAA_1\t$VARTYPE\t$Annotation\t$varLocation\t$exon\t$intron\tNA\t$Feature_ID:${HGVS.c}\t${HGVS.p}\tNA\tNA\tNA\t$rs\tNA\tNA\tNA\tNA\tNA\tNA\tNA\tNA\t$dbNSFP_1000Gp1_AF\t$dbNSFP_1000Gp1_AFR_AF\tNA\tNA\t$dbNSFP_1000Gp1_EUR_AF\t$dbNSFP_1000Gp1_AMR_AF\tNA\tNA\tNA\tNA\tNA\tNA\t$dbNSFP_ESP6500_EA_AF\t$dbNSFP_ESP6500_AA_AF\tNA\tNA\tNA\tNA\tNA\tNA\tNA\tNA\tNA\tNA\tNA\t$dbNSFP_SIFT_pred\tNA\tNA\t$dbNSFP_Polyphen2_HVAR_pred\tNA\t$dbNSFP_phastCons\tNA\tNA\tNA\tNA\tNA\tNA\tNA\tNA\tNA\tNA\tNA\tNA\tNA\tNA\tNA\tNA\tNA\tNA\tNA\tNA\tNA\tNA\tNA"	 
		    if {$Annotation == "sequence_feature"} {continue}
		    break
		    #puts "g_ANNOTATION($ID): $g_ANNOTATION($ID)"
		}
	    }
	    ## We need to extract the INFO column (additional SnpEff information) for each ID
	    ## Storing is done here but extraction is done while ranking to ensure the big picture
	    set lINFO [split $INFO ";"]
	    set i_ann [lsearch -regexp $lINFO "^ANN="]
	    if {$i_ann != -1} {
		set g_vcfINFOS_SnpEff($ID) [lindex $lINFO $i_ann]
	    } else {
		set g_vcfINFOS_SnpEff($ID) "NA"
	    }
	    set i_lof [lsearch -regexp $lINFO "^LOF="]
	    if {$i_lof != -1} {
		lappend g_vcfINFOS_SnpEff($ID) [lindex $lINFO $i_lof]
	    } else {
		lappend g_vcfINFOS_SnpEff($ID) "NA"
	    }
	    set i_nmd [lsearch -regexp $lINFO "^NMD="]
	    if {$i_nmd != -1} {
		lappend g_vcfINFOS_SnpEff($ID) [lindex $lINFO $i_nmd]
	    } else {
		lappend g_vcfINFOS_SnpEff($ID) "NA"
	    }
	    
	}
	if {[info exists g_VaRank(Annotation)]} {
	    set g_VaRank(Annotation) [lsort -unique $g_VaRank(Annotation)]
	}
	if {[info exists g_VaRank(DEBUG)]} {
	    puts "L_Coding_effect: $g_VaRank(Annotation)"
	}
    }

    set g_vcfINFOS_SnpEff(Header) "SnpEff_ANN SnpEff_LOF SnpEff_NMD"

    return 
}

proc correspondanceAA3inAA1 {AA3} {
    set c(Gly) G
    set c(Ala) A
    set c(Val) V 
    set c(Leu) L
    set c(Ile) I
    set c(Met) M
    set c(Phe) F
    set c(Trp) W
    set c(Pro) P
    set c(Ser) S
    set c(Thr) T
    set c(Cys) C
    set c(Tyr) Y
    set c(Asn) N
    set c(Gln) Q
    set c(Asp) D
    set c(Glu) E
    set c(Lys) K
    set c(Arg) R
    set c(His) H
    set c(*) *
    
    if {[info exists c($AA3)]} {return $c($AA3)} else {return "NA"}
}
